#!/usr/bin/python3

import datetime
import numpy as np
import netCDF4
import sys
sys.path.append('../')
sys.path.append('.')
import pyLARDA.helpers as h
import pyLARDA.Transformations as Transf
import pyLARDA.NcReader as NcReader

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

import logging
logger = logging.getLogger(__name__)


try:
    # compile and load the c-version
    #import pyximport
    #pyximport.install()
    import pyLARDA.peakTree_fastbuilder as peakTree_fastbuilder
    fastbuilder = True
except:
    # use the numpy only version
    fastbuilder = False
    #logger.exception("compiling peakTree_fastbuilder failed, using numpy version")

def build_tree_py(data, ldr_avail):
    """pure python/numpy version of the build tree function
    (slower than the compiled cython version)

    Indices in the stacked data array:

    .. code::
    
        0: 'parent', 1: 'Z', 2: 'v',
        3: 'width', 4: 'skew', 5: 'threshold',
        6: 'prominence', 7: 'bound_l', 8: 'bound_r',
        optional
        9: 'LDR', 10: 'ldrmax'
    """
    
    travtree = {}            
    #logger.debug('peakTree parent {}'.format(ncD.variables['parent'][it,ir,:]))
    parent = np.ma.masked_less(data[:,0], -990)
    avail_nodes = np.argwhere(parent > -10).ravel()
    #print(data[:,0].mask, type(data[:,0]), parent, avail_nodes)
    for k in avail_nodes.tolist():
        node = {'parent_id': np.asscalar(data[k,0]), 
                'thres': np.asscalar(data[k,5]), 
                'width': np.asscalar(data[k,3]), 
                'z': np.asscalar(data[k,1]), 
                'bounds': (np.asscalar(data[k,7]), np.asscalar(data[k,8])),
                #'coords': [0], 
                'skew': np.asscalar(data[k,4]),
                'prominence': np.asscalar(data[k,6]),
                'v': np.asscalar(data[k,2])}
        node['id'] = k
        node['bounds'] = list(map(int, node['bounds']))
        node['width'] = node['width'] if np.isfinite(node['width']) else -99
        node['skew'] = node['skew'] if np.isfinite(node['skew']) else -99
        node['thres'] = node['thres'] if np.isfinite(node['thres']) else -99
        node['prominence'] = node['prominence'] if np.isfinite(node['prominence']) else -99
        if ldr_avail:
            node['ldr'] = np.asscalar(data[k,9]) 
            node['ldr'] = node['ldr'] if np.isfinite(node['ldr']) else -99
            node['ldrmax'] = np.asscalar(data[k,10])
            node['ldrmax'] = node['ldrmax'] if np.isfinite(node['ldrmax']) else -99
        else:
            node['ldr'], node['ldrmax'] = -99, -99
        if node['parent_id'] != -999:
            if k == 0:
                node['coords'] = [0]
            else:
                coords = travtree[node['parent_id']]['coords']
                if k%2 == 0:
                    node['coords'] = coords + [1]
                else:
                    node['coords'] = coords + [0]
            
            # remove the parent id for compatibility to peakTreeVis
            if node['parent_id'] == -1:
                del node['parent_id']
            # format for transport
            #v = {ky: format_for_json(val) for ky, val in v.items()}
            travtree[k] = node
    return travtree


def array_to_tree_py(data, ldr_avail):
    """convert the array from the netcdf to var of data container
    pure python/numpy version
    (slower than the compiled cython version)
    """
    trees = np.empty((data.shape[0], data.shape[1]), dtype=object)
    mask = np.empty((data.shape[0], data.shape[1]), dtype=bool)
    mask[:] = True
    
    for it in range(data.shape[0]):
        for ir in range(data.shape[1]):
            trees[it, ir] = build_tree_py(data[it,ir,:,:], ldr_avail)
            mask[it, ir] = False
            
    return trees, mask


def peakTree_reader(paraminfo):
    """build a function for reading the peakTree data (setup by connector)"""
    def pt_ret(f, time_interval, *further_intervals):
        """function that converts the peakTree netCDF to the data container
        """
        logger.debug("filename at reader {}".format(f))
        with netCDF4.Dataset(f, 'r') as ncD:

            times = ncD.variables[paraminfo['time_variable']][:].astype(np.float64)
            if 'time_millisec_variable' in paraminfo.keys() and \
                    paraminfo['time_millisec_variable'] in ncD.variables:
                subsec = ncD.variables[paraminfo['time_millisec_variable']][:]/1.0e3
                times += subsec
            if 'time_microsec_variable' in paraminfo.keys() and \
                    paraminfo['time_microsec_variable'] in ncD.variables:
                subsec = ncD.variables[paraminfo['time_microsec_variable']][:]/1.0e6
                times += subsec

            timeconverter, _ = h.get_converter_array(
                paraminfo['time_conversion'], ncD=ncD)
            ts = timeconverter(times)

            #print('timestamps ', ts[:5])
            # setup slice to load base on time_interval
            it_b = h.argnearest(ts, h.dt_to_ts(time_interval[0]))
            if len(time_interval) == 2:
                it_e = h.argnearest(ts, h.dt_to_ts(time_interval[1]))
                if ts[it_e] < h.dt_to_ts(time_interval[0])-3*np.median(np.diff(ts)):
                    logger.warning(
                            'last profile of file {}\n at {} too far from {}'.format(
                                f, h.ts_to_dt(ts[it_e]), time_interval[0]))
                    return None

                it_e = it_e+1 if not it_e == ts.shape[0]-1 else None
                slicer = [slice(it_b, it_e)]
            else:
                slicer = [slice(it_b, it_b+1)]
            print(slicer)

            if paraminfo['ncreader'] == 'peakTree':
                range_tg = True

                range_interval = further_intervals[0]
                ranges = ncD.variables[paraminfo['range_variable']]
                logger.debug('loader range conversion {}'.format(paraminfo['range_conversion']))
                rangeconverter, _ = h.get_converter_array(
                    paraminfo['range_conversion'],
                    altitude=paraminfo['altitude'])
                ir_b = h.argnearest(rangeconverter(ranges[:]), range_interval[0])
                if len(range_interval) == 2:
                    if not range_interval[1] == 'max':
                        ir_e = h.argnearest(rangeconverter(ranges[:]), range_interval[1])
                        ir_e = ir_e+1 if not ir_e == ranges.shape[0]-1 else None
                    else:
                        ir_e = None
                    slicer.append(slice(ir_b, ir_e))
                else:
                    slicer.append(slice(ir_b, ir_b+1))

            varconverter, maskconverter = h.get_converter_array(
                paraminfo['var_conversion'])

            its = np.arange(ts.shape[0])[tuple(slicer)[0]]
            irs = np.arange(ranges.shape[0])[tuple(slicer)[1]]
            var = np.empty((its.shape[0], irs.shape[0]), dtype=object)
            mask = np.empty((its.shape[0], irs.shape[0]), dtype=bool)
            mask[:] = True

            param_list = [
                ncD.variables['parent'][tuple(slicer)[0],tuple(slicer)[1],:], #0
                ncD.variables['Z'][tuple(slicer)[0],tuple(slicer)[1],:],      #1
                ncD.variables['v'][tuple(slicer)[0],tuple(slicer)[1],:],      #2
                ncD.variables['width'][tuple(slicer)[0],tuple(slicer)[1],:],  #3
                ncD.variables['skew'][tuple(slicer)[0],tuple(slicer)[1],:],   #4
                ncD.variables['threshold'][tuple(slicer)[0],tuple(slicer)[1],:], #5
                ncD.variables['prominence'][tuple(slicer)[0],tuple(slicer)[1],:], #6
                ncD.variables['bound_l'][tuple(slicer)[0],tuple(slicer)[1],:],    #7
                ncD.variables['bound_r'][tuple(slicer)[0],tuple(slicer)[1],:]     #8
            ]
            if 'LDR' in ncD.variables.keys():
                ldr_avail = True
                param_list.append(ncD.variables['LDR'][tuple(slicer)[0],tuple(slicer)[1],:])  #9
                param_list.append(ncD.variables['ldrmax'][tuple(slicer)[0],tuple(slicer)[1],:]) #10
            else:
                ldr_avail = False
            data = np.stack(tuple(param_list), axis=3)
            print(data.shape)
            if fastbuilder:
                var, mask = peakTree_fastbuilder.array_to_tree_c(data.astype(float), ldr_avail)
            else:
                var, mask = array_to_tree_py(data, ldr_avail)

            data = {}
            data['dimlabel'] = ['time', 'range', 'dict']

            data["filename"] = f
            data["paraminfo"] = paraminfo
            data['ts'] = ts[tuple(slicer)[0]]

            data['system'] = paraminfo['system']
            data['name'] = paraminfo['paramkey']
            data['colormap'] = paraminfo['colormap']

            if 'meta' in paraminfo:
                data['meta'] = NcReader.get_meta_from_nc(ncD, paraminfo['meta'], paraminfo['variable_name'])

            data['rg'] = rangeconverter(ranges[tuple(slicer)[1]])
            data['rg_unit'] = NcReader.get_var_attr_from_nc("identifier_rg_unit", 
                                                paraminfo, ranges)
            logger.debug('shapes {} {} {}'.format(ts.shape, ranges.shape, var.shape))

            logger.debug('shapes {} {}'.format(ts.shape, var.shape))
            data['var_unit'] = NcReader.get_var_attr_from_nc("identifier_var_unit", 
                                                    paraminfo, var)
            data['var_lims'] = [float(e) for e in \
                                NcReader.get_var_attr_from_nc("identifier_var_lims", 
                                                    paraminfo, var)]

            data['var'] = varconverter(var)
            data['mask'] = maskconverter(mask)

            return data

    return pt_ret


def tree_to_timeheight(data_cont, param, sel_node=0, **kwargs):
    """convert the tree data container to a normal time-height data container by extracting a node and a parameter
    
    Args:
        data_cont (dict): data container
        param (str): parameter to select, eg z, v
        sel_node (np.array or int, optional): integer or array of index to select
        **kwargs


    Returns:
        data container of dimension ``['time', 'range']``
    """
    assert data_cont['dimlabel'] == ['time', 'range', 'dict'], "dimlabel does not match"
    if type(sel_node) is not int:
        assert data_cont['var'].shape == sel_node.shape
        sel_nodes_array = True
    else:
        sel_nodes_array = False
    
    new_cont = data_cont.copy()
    var = np.empty(data_cont['var'].shape, dtype=float)
    var[:] = np.nan
    mask = np.empty(data_cont['var'].shape, dtype=bool)
    mask[:] = True
    for index, tree in np.ndenumerate(data_cont['var']):
        #print(index, sel_nodes[index])
        if param == 'no_nodes':
            var[index] = len(data_cont['var'][index].keys())
            mask[index] = False
        elif sel_nodes_array:
            if not sel_node[index] == -1:
                var[index] = data_cont['var'][index][sel_node[index]][param]
                mask[index] = False
        else:
            if sel_node in data_cont['var'][index]:
                var[index] = data_cont['var'][index][sel_node][param]
                mask[index] = False
            
            
    new_cont['var'] = var
    new_cont['mask'] = mask
    new_cont['dimlabel'] = ['time', 'range']
    new_cont['colormap'] = 'jet'
    new_cont['var_lims'] = [-50, 10]
    if 'var_units' in kwargs:
        new_cont['var_units'] = kwargs
    return new_cont


def select_rimed_node(data_cont):
    """select the rimed nodes from a peaktree data container
    The nodes are filtered by the rule:
    ``abs(n['v'][0]-n['v'][-1]) > 1.5``

    Args:
        data_cont: peakTree data container

    Returns:
        data_container with selected indices in ``var`` of shape ``(time, range)``
    """

    new_cont = {**data_cont}
    var = np.empty(data_cont['var'].shape, dtype=int)
    var[:] = -1
    for index, tree in np.ndenumerate(data_cont['var']):
        nodes = list(tree.values())
        if nodes:
            nodes.sort(key=lambda n: n['v'])
            if len(nodes) > 1:
                if abs(nodes[0]['v'] - nodes[-1]['v']) > 1.5:
                    var[index] = nodes[0]['id']
                #print(index, nodes)
            #print(index, len(nodes))
            #print(k, ' filtered nodes ', len(nodes), [(e['id'], e['z'], e['v']) for e in nodes])

    new_cont['var'] = var
    new_cont['mask'] = (var == -1)
    new_cont['name'] = 'selected index'
    new_cont['dimlabel'] = ['time', 'range']
    new_cont['var_unit'] = ''
    return new_cont


def select_liquid_node(data_cont, **kwargs):
    """select the liquid nodes from a peaktree data container

    The nodes are filtered by the rule:
    ``n['z'] < -20 and abs(n['v']) < 0.3``

    Args:
        data_cont: peakTree data container

    Key word arguments:
        Z_thresh, maximum Z of liquid peak (default is -20)
        LDR_thresh, maximum LDR of liquid peaks (LDR ignored if not given)

    Returns:
        data_container with selected indices in ``var`` of shape ``(time, range)``
    """
    Z_thresh = kwargs['Z_thresh'] if 'Z_thresh' in kwargs else -20
    LDR_thresh = kwargs['LDR_thresh'] if 'LDR_thresh' in kwargs else False
    new_cont = {**data_cont}
    var = np.empty(data_cont['var'].shape, dtype=int)
    var[:] = -1
    for index, tree in np.ndenumerate(data_cont['var']):
        if not LDR_thresh:
            nodes = list(filter(lambda n: n['z'] < Z_thresh and abs(n['v']) < 0.3, tree.values()))
        else:
            nodes = list(filter(lambda n: n['z'] < Z_thresh and abs(n['v']) < 0.3 and n['ldr'] < LDR_thresh,
                                tree.values()))
        if nodes:
            nodes.sort(key=lambda n: n['v'])
            if len(nodes) > 1:
                pass
                # print(index, nodes)
            # print(index, len(nodes))
            # print(k, ' filtered nodes ', len(nodes), [(e['id'], e['z'], e['v']) for e in nodes])
            var[index] = nodes[-1]['id']

    new_cont['var'] = var
    new_cont['mask'] = (var == -1)
    new_cont['name'] = 'selected index'
    new_cont['dimlabel'] = ['time', 'range']
    new_cont['var_unit'] = ''
    return new_cont


def select_fastest_node(data_cont):
    """select the fastest-falling nodes from a peaktree data container

    Args:
        data_cont: peakTree data container

    Returns:
        data_container with selected indices in ``var`` of shape ``(time, range)``
    """

    new_cont = {**data_cont}
    var = np.empty(data_cont['var'].shape, dtype=int)
    var[:] = -1
    for index, tree in np.ndenumerate(data_cont['var']):
        if tree:
            fastest = min([x['v'] for x in tree.values()])
            nodes = list(filter(lambda n: n['v'] == fastest, tree.values()))
        else: nodes = []
        if nodes:
            nodes.sort(key=lambda n: n['id'])
            if len(nodes) > 1:
                pass
                # print(index, nodes)
            # print(index, len(nodes))
            # print(k, ' filtered nodes ', len(nodes), [(e['id'], e['z'], e['v']) for e in nodes])
            var[index] = nodes[0]['id']

    new_cont['var'] = var
    new_cont['mask'] = (var == -1)
    new_cont['name'] = 'selected index'
    new_cont['dimlabel'] = ['time', 'range']
    new_cont['var_unit'] = ''
    return new_cont


def plot_no_nodes(data_cont, **kwargs):
    """wrapper for :py:mod:`pyLARDA.Transformations.plot_timeheight2` to plot the no of nodes

    Args:
        data (dict): data container
        **kwargs: piped to plot_timeheight2 function

    Returns:
        ``fig, ax``
    """
    #data_cont['colormap'] = matplotlib.colors.ListedColormap(
    #    ["#ffffff", "#cdbfbc", "#987b61", "#fdff99", "#35d771", "#1177dd"], 'terrain_seq')
    data_cont['colormap'] = matplotlib.colors.ListedColormap(
    ["#ffffff", "#cccccc", "#cc6677", "#88ccee", "#eecc66", "#332288"], 'pTcat')
    # We must be sure to specify the ticks matching our target names
    #labels = {0: '0', 1: "1", 2: "3", 3: "5", 4: "7", 5: "9"}
    labels = {0: ' 0', 1: " 1", 2: " 2", 3: " 3", 4: " 4", 5: " 5"}

    data_cont['name'] = 'no. peaks'
    cbarformatter = plt.FuncFormatter(lambda val, loc: labels[val])
    data_cont['var'] = np.ceil(np.array(data_cont['var'])/2.)
    data_cont["var_lims"] = [-0.5, 5.5]
    fig, ax = Transf.plot_timeheight2(data_cont, **kwargs)
    #ax.cbar.set_yticks([0, 1, 2, 3, 4, 5])
    cbar = fig.axes[1]
    #cbar.set_ylabel("Number of nodes", fontweight='semibold', fontsize=15)
    #print(fig.axes[1].get_yticks())
    #print(cbar.ax.get_ticklocs())
    #print(fig.axes)
    #print(fig.axes[1])
    cbar_ylabel = ax.images[0].colorbar.ax.get_ylabel()
    ax.images[0].colorbar.ax.set_ylabel(cbar_ylabel[:-2])
    fig.axes[1].set_yticklabels(labels.values())
    return fig, ax


def plot_sel_index(data_cont, **kwargs):
    """wrapper for :py:mod:`pyLARDA.Transformations.plot_timeheight2` to plot the index of selected node

    Args:
        data (dict): data container
        **kwargs: piped to plot_timeheight2 function

    Returns:
        ``fig, ax``
    """
    data_cont['colormap'] = plt.cm.get_cmap('jet', 7)
    # We must be sure to specify the ticks matching our target names

    data_cont['mask'] = (data_cont['var'] < 0)
    data_cont["var_lims"] = [-0.5, 6.5]
    fig, ax = Transf.plot_timeheight2(data_cont, **kwargs)
    #ax.cbar.set_yticks([0, 1, 2, 3, 4, 5])
    cbar = fig.axes[1]
    #cbar.set_ylabel("Number of nodes", fontweight='semibold', fontsize=15)
    #print(fig.axes[1].get_yticks())
    #print(cbar.ax.get_ticklocs())
    #print(fig.axes)
    #print(fig.axes[1])
    #fig.axes[1].set_yticklabels(labels.values())
    return fig, ax



def child_iter(indices):
    def children(i):
        "helper to geht the correct order"
        if i in indices:
            yield i
            yield from children(2*i+1)
            yield from children(2*i+2)
    return children



def to_text(data_cont):
    """represent the tree as a multiline string"""
    maxind_at_level = np.cumsum([2**n for n in range(5)]) - 1

    single_tree = data_cont['var']
    no_levels = np.searchsorted(maxind_at_level, max(single_tree.keys()))
    print("no levels", no_levels)

    dt = h.ts_to_dt(data_cont['ts']) 
    lines = []
    lines.append("tree at ts {} and rg {:.2f}".format(
        dt.strftime("%Y-%m-%d %H:%M:%S.%f")[:-3], data_cont['rg']))
    header = "id (bounds)  " + (no_levels)*"   " + \
       "        Z       v      σ      𝛾"
    if single_tree[0]['ldr'] != -99:
        header += "    ldr ldrmax"
    header += "  thres   prom"
    lines.append(header)
    
    # iterate over children depth first, to build a proper tree
    for i in child_iter(list(single_tree.keys()))(0):
        moms = single_tree[i]
        #print(i, moms["bounds"])
        level = np.searchsorted(maxind_at_level, i)
        spc_bef = ""
        if level > 0:
            if np.floor((i-1)/2.)%2 or np.floor((i-1)/2.) == 0:
                if i%2:
                    spc_bef = " " + (level-1)*"|  " + "+-"
                else:
                    spc_bef = " " + (level-1)*"|  " + "`-"
            else:
                if i%2:
                    spc_bef = " " + (level-2)*"|  " + "   +-"
                else:
                    spc_bef = " " + (level-2)*"|  " + "   `-"
        spc_aft = (no_levels-level)*"   "
        bounds_f = '({:>3d}, {:>3d})'.format(*moms['bounds'])

        momstr = '{:> 6.2f}, {:> 6.2f}, {:>5.2f}, {:> 3.2f}'.format(
            moms['z'], moms['v'], moms['width'], moms['skew'])
        if moms['ldr'] != -99:
            momstr += ", {:> 5.1f}, {:> 5.1f}".format(moms['ldr'], moms['ldrmax'])
        momstr += ", {:> 5.1f}, {:> 5.1f}".format(moms['thres'], moms['prominence'])
        txt = "{}{:>2d} {}{} | {}".format(spc_bef, i, bounds_f, spc_aft, momstr)
        lines.append(txt)
    return '\n'.join(lines)


def print_tree(data_cont):
    """print the tree as a multi line string"""
    print(to_text(data_cont))



#!/usr/bin/python3

import sys, os
# just needed to find pyLARDA from this location
sys.path.append('../')

import datetime
import pprint
import toml
import time, json
#import cbor    #library for binary data transfer
import msgpack #try the messagepack binary format 

import pyLARDA
import pyLARDA.helpers as h
from flask import Flask, jsonify, request, Response, send_file, redirect
from flask_cors import CORS
#from flask_compress import Compress
import traceback

import numpy as np

import logging
import logging.handlers

app = Flask(__name__, static_url_path='', static_folder='public')
CORS(app)

#COMPRESS_MIMETYPES = ['application/json']
#COMPRESS_LEVEL = 18
#Compress(app)


app.logger.setLevel(logging.DEBUG)
log_larda = logging.getLogger('pyLARDA')
log_larda.setLevel(logging.INFO)
log_w = logging.getLogger('werkzeug')
log_w.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s [%(process)d] %(name)s %(levelname)s: %(message)s')

#can be used when migrated to pyLARDA
#ROOT_DIR = os.path.dirname(os.path.abspath(__file__)) 

print(logging.Logger.manager.loggerDict.keys())
if 'gunicorn' in logging.Logger.manager.loggerDict.keys():
    print('found gunicorn logger')
    gunicorn_e = logging.getLogger('gunicorn.error')
    print(gunicorn_e.name, gunicorn_e.level, gunicorn_e.handlers)
    gunicorn_a = logging.getLogger('gunicorn.access')
    gunicorn_a.setLevel(logging.DEBUG)
    print(gunicorn_a.name, gunicorn_a.level, gunicorn_a.handlers)
    for handler in gunicorn_a.handlers:
        print(handler, handler.level)
        handler.setFormatter(formatter)
        app.logger.addHandler(handler)
        log_w.addHandler(handler)
        log_larda.addHandler(handler)
else:
    print('no gunicorn logger')
    fh = logging.handlers.RotatingFileHandler("larda_from_flask.log", maxBytes=10e6, backupCount=3)
    fh.setFormatter(formatter)

    app.logger.addHandler(fh)
    log_w.addHandler(fh)
    log_larda.addHandler(fh)


@app.errorhandler(500)
def page_not_found(error):
    exc_info = sys.exc_info()
    return Response(json.dumps(traceback.format_exc()), status=500, mimetype='application/json')


@app.route('/')
def root():
    #return app.send_static_file('index.html')
    # necessary to return location header
    resp = send_file('public/index.html')
    #resp.headers["Location"] = '/'
    return resp
    #return redirect('/larda3/data_avail')


@app.route('/explorer/<camp>')
def explorer(camp):
    #return app.send_static_file('index.html')
    # necessary to return location header
    resp = send_file('public/explorer.html')
    #resp.headers["Location"] = '/'
    return resp
    #return redirect('/larda3/data_avail')

@app.route('/api/', methods=['GET'])
def api_entry():
    """

    Returns:
        campaign list
    """
    larda = pyLARDA.LARDA()
    return jsonify(campaign_list=larda.campaign_list)


@app.route('/api/<campaign_name>/', methods=['GET'])
def get_campaign_info(campaign_name):
    """

    Returns:
        json object with all the campaign info
    """
    campaign_info = {}
    larda=pyLARDA.LARDA().connect(campaign_name, build_lists=False)

    campaign_info['config_file'] = larda.camp.info_dict
    #print("Parameters in stock: ",[(k, larda.connectors[k].params_list) for k in larda.connectors.keys()])
    campaign_info['info_text'] = larda.camp.INFO_TEXT
    
    campaign_info['connectors'] = {system:conn.get_as_plain_dict() for system, conn in larda.connectors.items()}

    return jsonify(**campaign_info)


@app.route('/api/<campaign_name>/<system>/<param>', methods=['GET'])
def get_param(campaign_name, system, param):
    """ """
    app.logger.info("got request for {} {} {}".format(campaign_name, system, param))
    starttime = time.time()
    larda=pyLARDA.LARDA().connect(campaign_name, build_lists=False)
    app.logger.debug("{:5.3f}s load larda".format(time.time() - starttime))

    if "rformat" in request.args and request.args['rformat'] == 'bin':
        rformat = 'bin'
    elif "rformat" in request.args and request.args['rformat'] == 'msgpack':
        rformat = 'msgpack'
    else:
        rformat = 'json'
    intervals = request.args.get('interval').split(',') 
    time_interval = [h.ts_to_dt(float(t)) for t in intervals[0].split('-')]
    further_slices = [[float(e) if e not in ['max'] else e for e in s.split('-')] for s in intervals[1:]]
    
    app.logger.warning('request.args {}'.format(dict(request.args)))
    app.logger.info("time request {}".format(time_interval))
    starttime = time.time()
    data_container = larda.read(system, param, time_interval, *further_slices, **dict(request.args))
    app.logger.debug("{:5.3f}s read data".format(time.time() - starttime))
    starttime = time.time()
    #for k in data_container.keys():
    #    app.logger.warning(f'{k} {type(data_container[k])}')
    for k in ['ts', 'rg', 'vel', 'var', 'mask', 'vel_ch2', 'vel_ch3']:
        if k in data_container and hasattr(data_container[k], 'tolist'):
            if data_container[k].dtype is not np.dtype('object'):
                data_container[k][~np.isfinite(data_container[k])] = 0
            data_container[k] = data_container[k].tolist()
        #if k in data_container:
        #    app.logger.warning(f'{k} {type(data_container[k])}')
    #for k in data_container.keys():
    #    app.logger.warning(f'{k} {type(data_container[k])}')
    app.logger.debug("{:5.3f}s convert data".format(time.time() - starttime))

    #import io
    #test_datacont = {**data_container}
    #for k in ['ts', 'rg', 'var', 'mask']:
    #    if k in test_datacont:
    #        print(k,type(test_datacont[k]))
    #        start = time.time()
    #        test_datacont[k][~np.isfinite(test_datacont[k])] = 0
    #        memfile = io.BytesIO()
    #        np.save(memfile, test_datacont[k])
    #        memfile.seek(0)
    #        test_datacont[k] = memfile.read().decode('latin-1')
    #resp = Response(json.dumps(test_datacont), status=200, mimetype='application/json')

    starttime = time.time()
    #if rformat == 'bin':
    #    resp = Response(cbor.dumps(data_container), status=200, mimetype='application/cbor')
    if rformat == 'msgpack':
        resp = Response(msgpack.packb(data_container), status=200, mimetype='application/msgpack')
    elif rformat == 'json':
        resp = Response(json.dumps(data_container), status=200, mimetype='application/json')
    app.logger.debug("{:5.3f}s dumps {}".format(time.time() - starttime, rformat))

    #for some reason the manual response is faster...
    #return jsonify(data_container)
    return resp

@app.route('/description/<campaign_name>/<system>/<parameter>', methods=['GET'])
def get_descript(campaign_name, system, parameter):
    """ """
    app.logger.info("got request fori description {} {} {}".format(campaign_name, system, parameter))

    larda=pyLARDA.LARDA().connect(campaign_name, build_lists=False)

    #if "rformat" in request.args and request.args['rformat'] == 'bin':
    #    rformat = 'bin'
    #elif "rformat" in request.args and request.args['rformat'] == 'msgpack':
    #    rformat = 'msgpack'
    #else:
    #    rformat = 'json'
    
    app.logger.warning('request.args {}'.format(dict(request.args)))
    text_string = larda.description(system, parameter)

    #if rformat == 'bin':
    #    resp = Response(cbor.dumps(data_container), status=200, mimetype='application/cbor')
    #elif rformat == 'msgpack':
    #    resp = Response(msgpack.packb(data_container), status=200, mimetype='application/msgpack')
    #elif rformat == 'json':
    #    resp = Response(json.dumps(data_container), status=200, mimetype='application/json')
    resp = Response(text_string.encode('utf-8'), status=200, mimetype='text/plain')

    return resp

@app.route('/peakTree')
def peakTree():
    return app.send_static_file('peakTreeVis.html')




"""
This routine calculates the radar moments for the RPG 94 GHz FMCW radar 'LIMRAD94' and generates a NetCDF4 file.
The generated files can be used as input for the Cloudnet processing chain.

Args:
    **date (string): format YYYYMMDD
    **path (string): path where NetCDF file will be stored

Example:
    python spec2mom_limrad94.py date=20181201 path=/tmp/pycharm_project_626/scripts_Willi/cloudnet_input/

"""

import sys, datetime, time

sys.path.append('../')
sys.path.append('.')

import pyLARDA
import pyLARDA.helpers as h
import pyLARDA.NcWrite as nc
from larda.pyLARDA.spec2mom_limrad94 import spectra2moments, build_extended_container

import logging

import numpy as np

########################################################################################################################
#
#
#   _______ _______ _____ __   _       _____   ______  _____   ______  ______ _______ _______
#   |  |  | |_____|   |   | \  |      |_____] |_____/ |     | |  ____ |_____/ |_____| |  |  |
#   |  |  | |     | __|__ |  \_|      |       |    \_ |_____| |_____| |    \_ |     | |  |  |
#
#
if __name__ == '__main__':

    start_time = time.time()

    log = logging.getLogger('pyLARDA')
    log.setLevel(logging.INFO)
    log.addHandler(logging.StreamHandler())

    # Load LARDA

    # larda = pyLARDA.LARDA('remote', uri='http://larda.tropos.de/larda3').connect('lacros_dacapo', build_lists=False)
    larda = pyLARDA.LARDA().connect('lacros_dacapo_catalpa')
    c_info = [larda.camp.LOCATION, larda.camp.VALID_DATES]

    # print('available systems:', larda.connectors.keys())
    # print("available parameters: ", [(k, larda.connectors[k].params_list) for k in larda.connectors.keys()])
    print('days with data', larda.days_with_data())

    # gather command line arguments
    method_name, args, kwargs = h._method_info_from_argv(sys.argv)

    # gather argument
    if 'date' in kwargs:
        date = str(kwargs['date'])
        begin_dt = datetime.datetime.strptime(date + ' 00:00:05', '%Y%m%d %H:%M:%S')
        end_dt = datetime.datetime.strptime(date + ' 23:59:55', '%Y%m%d %H:%M:%S')
    else:
        date = '20190918'
        begin_dt = datetime.datetime.strptime(date + ' 00:00:05', '%Y%m%d %H:%M:%S')
        end_dt = datetime.datetime.strptime(date + ' 23:59:55', '%Y%m%d %H:%M:%S')

    std_above_mean_noise = float(kwargs['NF']) if 'NF' in kwargs else 6.0

    LIMRAD_Zspec = build_extended_container(larda, 'VSpec', begin_dt, end_dt,
                                            rm_precip_ghost=True,
                                            do_despeckle3d=False,
                                            estimate_noise=True,
                                            noise_factor=6.0
                                            )

    LIMRAD94_moments = spectra2moments(LIMRAD_Zspec, larda.connectors['LIMRAD94'].system_info['params'],
                                       despeckle=True,
                                       main_peak=True,
                                       filter_ghost_C1=True)

    ########################################################################################################################
    #
    #   _ _ _ ____ _ ___ ____    ____ ____ _    _ ___  ____ ____ ___ ____ ___     _  _ ____    ____ _ _    ____
    #   | | | |__/ |  |  |___    |    |__| |    | |__] |__/ |__|  |  |___ |  \    |\ | |       |___ | |    |___
    #   |_|_| |  \ |  |  |___    |___ |  | |___ | |__] |  \ |  |  |  |___ |__/    | \| |___    |    | |___ |___
    #
    for var in ['DiffAtt', 'ldr', 'bt', 'rr', 'LWP', 'MaxVel', 'C1Range', 'C2Range', 'C3Range', 'SurfRelHum']:
        print('loading variable from LV1 :: ' + var)
        LIMRAD94_moments.update({var: larda.read("LIMRAD94", var, [begin_dt, end_dt], [0, 'max'])})
    LIMRAD94_moments['DiffAtt']['var'] = np.ma.masked_where(LIMRAD94_moments['Ze']['mask'] == True,
                                                            LIMRAD94_moments['DiffAtt']['var'])
    LIMRAD94_moments['ldr']['var'] = np.ma.masked_where(LIMRAD94_moments['Ze']['mask'] == True,
                                                        LIMRAD94_moments['ldr']['var'])

    cloudnet_remsens_lim_path = '/media/sdig/LACROS/cloudnet/data/'

    if 'path' in kwargs:
        path = kwargs['path']
    else:
        if c_info[0] == 'Punta Arenas':
            path = cloudnet_remsens_lim_path + 'punta-arenas-limrad/' + 'calibrated/limrad94/' + date[:4] + '/'
        elif c_info[0] == 'Leipzig':
            path = cloudnet_remsens_lim_path + 'leipzig/' + 'calibrated/limrad94/' + date[:4] + '/'
        elif c_info[0] == 'RV-Meteor':
            path = cloudnet_remsens_lim_path + 'leipzig/' + 'calibrated/limrad94/' + date[:4] + '/'
        else:
            print('Error: No other sites implemented jet!')
            sys.exit(-42)

    flag = nc.generate_cloudnet_input_LIMRAD94(LIMRAD94_moments, path)

    ########################################################################################################################

    print('total elapsed time = {:.3f} sec.'.format(time.time() - start_time))




#!
# Copyright (c) 2017 Siphon Contributors.
# Distributed under the terms of the BSD 3-Clause License.
# SPDX-License-Identifier: BSD-3-Clause
"""
Wyoming Upper Air Data Request
==============================

This routine gathers sounding data from the Wyoming upper air archive and generates two text files containing sounding
data and meta data, also a png of the sounding is provided.

This example shows how to use siphon's `simplewebswervice` support to create a query to
the Wyoming upper air archive.


necessary packages:

       conda install -c conda-forge metpy
       conda install -c conda-forge siphon


Args:
    **date (integer): format YYYYMMDD
    **hour (integer): e.g. 12
    **sation (string): station identifier, e.g. Punta-Arenas: SCCI
    **folder (string): path to folder where output is saved, default save where script is executed

Return:
    [date]_[hour].png:          plot of sounding
    [date]_[hour]_metadata.txt: meta data of sounding
    [date]_[hour]_sounding.txt: actual data of sounding


Example:
    python download_plot_sounding.py date=20181214 hour=12 station=SCCI

"""

import sys
sys.path.append('../')
sys.path.append('.')

from datetime import datetime
import matplotlib

matplotlib.use('Agg')
import matplotlib.pyplot as plt

from metpy.units import units
import metpy.calc as mpcalc
from metpy.plots import Hodograph, SkewT
import pyLARDA
import pyLARDA.helpers as h
import pyLARDA.wyoming as w

####################################################
# Create a datetime object for the sounding and string of the station identifier.
# gather arguments
station = 'SCCI'    # punta arenas chile
station = 'EHDB'    # nehterlands near cabauw, hours possible: 00, 12

method_name, args, kwargs = h._method_info_from_argv(sys.argv)

if 'date' in kwargs:
    date_str = str(kwargs['date'])
    year, month, day = int(date_str[:4]), int(date_str[4:6]), int(date_str[6:])
else:
    year, month, day = 2019, 3, 15

station     = kwargs['station'] if 'station' in kwargs else 'SCII'
hour        = int(kwargs['hour'])    if 'hour'    in kwargs else 12
output_path = kwargs['folder']  if 'folder'  in kwargs else ''

date = datetime(year, month, day, hour)

####################################################
# Make the request (a pandas dataframe is returned).
df = w.WyomingUpperAir.request_data(date, station)

# Drop any rows with all NaN values for T, Td, winds
# df.dropna(subset=('temperature', 'dewpoint', 'direction', 'speed',
#                       'u_wind', 'v_wind'), how='all').reset_index(drop=True)

##########################################################################

# We will pull the data out of the example dataset into individual variables and
# assign units.

p = df['pressure'].values * units.hPa
T = df['temperature'].values * units.degC
Td = df['dewpoint'].values * units.degC
wind_speed = df['speed'].values * units.knots
wind_dir = df['direction'].values * units.degrees
u, v = mpcalc.wind_components(wind_speed, wind_dir)
u_wind = df['u_wind'].values * units(df.units['u_wind'])
v_wind = df['v_wind'].values * units(df.units['v_wind'])

##########################################################################
# Thermodynamic Calculations
# --------------------------
#
# Often times we will want to calculate some thermodynamic parameters of a
# sounding. The MetPy calc module has many such calculations already implemented!
#
# * **Lifting Condensation Level (LCL)** - The level at which an air parcel's
#   relative humidity becomes 100% when lifted along a dry adiabatic path.
# * **Parcel Path** - Path followed by a hypothetical parcel of air, beginning
#   at the surface temperature/pressure and rising dry adiabatically until
#   reaching the LCL, then rising moist adiabatically.

# Calculate the LCL
lcl_pressure, lcl_temperature = mpcalc.lcl(p[0], T[0], Td[0])

print(lcl_pressure, lcl_temperature)

# Calculate the parcel profile.
parcel_prof = mpcalc.parcel_profile(p, T[0], Td[0]).to('degC')

##########################################################################
#  Skew-T Plotting
# ------------------------
#
# Fiducial lines indicating dry adiabats, moist adiabats, and mixing ratio are
# useful when performing further analysis on the Skew-T diagram. Often the
# 0C isotherm is emphasized and areas of CAPE and CIN are shaded.

# Create a new figure. The dimensions here give a good aspect ratio
fig = plt.figure(figsize=(9, 9))
skew = SkewT(fig, rotation=30)

# Plot the data using normal plotting functions, in this case using
# log scaling in Y, as dictated by the typical meteorological plot
skew.plot(p, T, 'r')
skew.plot(p, Td, 'g')
skew.plot_barbs(p, u, v)
skew.ax.set_ylim(1000, 100)
skew.ax.set_xlim(-40, 60)
skew.ax.set_xlabel('Temperature [°C]')
skew.ax.set_ylabel('Pressure [hPa]')

# Plot LCL temperature as black dot
skew.plot(lcl_pressure, lcl_temperature, 'ko', markerfacecolor='black')

# Plot the parcel profile as a black line
skew.plot(p, parcel_prof, 'k', linewidth=2)

# Shade areas of CAPE and CIN
skew.shade_cin(p, T, parcel_prof)
skew.shade_cape(p, T, parcel_prof)

# Plot a zero degree isotherm
skew.ax.axvline(0, color='c', linestyle='--', linewidth=2)

# Add the relevant special lines
skew.plot_dry_adiabats()
skew.plot_moist_adiabats()
skew.plot_mixing_lines()

# Add a title
plt.title(str(date) + ' ' + station + ' Punta Arenas')

# Add Legend
plt.legend(['Temperature', 'Dew Point', 'LCL', 'parcel profile'])

# Save the Figure and the data
filename = str(date.year) + str(date.month).zfill(2) + str(date.day).zfill(2) + '_' + str(date.hour) + '_' + station

file = output_path + filename + '.png'
fig.savefig(file, dpi=100, format='png')
plt.close()

df.to_csv(output_path + filename + '_sounding' + '.txt', sep='\t', index=None)

with open(output_path + filename + '_metadata' + '.txt', 'w') as f:
    for item in df._metadata:
        for item1, item2 in item.items():
            f.write(str(item1) + '\t' + str(item2) + '\n')

print('    Save File :: ' + file)
print('    Save File :: ' + output_path + filename + '_metadata' + '.txt')
print('    Save File :: ' + output_path + filename + '_sounding' + '.txt')



#!/usr/bin/python3

import re
from setuptools import setup
from Cython.Build import cythonize

with open('README.md') as f:
    readme = f.read()


#VERSIONFILE="pyLARDA/_version.py"
#verstrline = open(VERSIONFILE, "rt").read()
#VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
#mo = re.search(VSRE, verstrline, re.M)
#if mo:
#    verstr = mo.group(1)
#else:
#    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))
#from pyLARDA import __version__ as verstr
#from pyLARDA import __author__ as authorstr
meta = {}
with open("pyLARDA/_meta.py") as fp:
    exec(fp.read(), meta)

setup(
    name='pyLARDA',
    version=meta['__version__'],
    description='Data cube for handling atmospheric observations of profiling remote sensing instruments.',
    long_description=readme,
    long_description_content_type='text/markdown',
    author=meta['__author__'],
    author_email='radenz@tropos.de',
    url='https://github.com/lacros-tropos/larda',
    download_url='https://github.com/lacros-tropos/larda/archive/refs/tags/v3.3.tar.gz',
    license='MIT License',
    packages=['pyLARDA'],
    include_package_data=True,
    python_requires='>=3.8',
    # automatic installation of the dependencies did not work with the test.pypi
    # below the try to fix it
    setup_requires=['wheel', 'numpy==1.21', 'scipy>=1.6', 'netCDF4>=1.4.2', 'msgpack', 'cython>=0.29.13', 'xarray',
                      'matplotlib>=3.0.2', 'requests>=2.21', 'toml>=0.10.0', 'tqdm>=4.36.1', 'numba>=0.45.1'],
    install_requires=['numpy==1.21', 'scipy>=1.6', 'netCDF4>=1.4.2', 'msgpack', 'cython>=0.29.13', 'xarray',
                      'matplotlib>=3.0.2', 'requests>=2.21', 'toml>=0.10.0', 'tqdm>=4.36.1', 'numba>=0.45.1'],
    ext_modules=cythonize("pyLARDA/peakTree_fastbuilder.pyx"),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
    ],
)



#!/usr/bin/python3


import pyLARDA.Connector as Connector
import pyLARDA.ParameterInfo as ParameterInfo
import pyLARDA.spec2mom_limrad94 as spec2mom_limrad94
import datetime, os, calendar, copy, time
from pathlib import Path
import numpy as np
import csv
import logging
import toml
import requests
import json
import pprint

from pyLARDA._meta import __version__, __author__, __init_text__, __default_info__

ROOT_DIR = Path(__file__).resolve().parent
logger = logging.getLogger(__name__)

os.environ["HDF5_USE_FILE_LOCKING"] = 'FALSE'

class LARDA :
    """init a new larda instance

    Args:
        data_source (str, optional): either ``'local'`` or ``'remote'``
        uri: link to backend

    """
    def __init__(self, data_source='local', uri=None):
        if data_source == 'local':
            self.data_source = 'local'
            self.camp = LARDA_campaign(ROOT_DIR.parents[1] / "larda-cfg", 'campaigns.toml')
            self.campaign_list = self.camp.get_campaign_list()
        elif data_source == 'remote':
            self.data_source = 'remote'
            self.uri = uri
            resp = requests.get(self.uri + '/api/')
            self.campaign_list = resp.json()['campaign_list']
        logger.warning(__init_text__)
        logger.warning(f"campaign list: {', '.join(self.campaign_list)}")

    def connect(self, *args, **kwargs):
        "switch to decide whether connect to local or remote data source"
        if self.data_source == 'local':
            return self.connect_local(*args, **kwargs)
        elif self.data_source == 'remote':
            return self.connect_remote(*args, **kwargs)


    def connect_local(self, camp_name, build_lists=True):
        """built the connector list for the specified campaign (only valid systems are considered)
        the connectors are instances of the Connector.Connector Class

        NEW: one connector per system
        then the params are parts of this system

        Args:
            camp_name (str): name of campaign as defined in ``campaigns.toml``
            build_lists (Bool, optional): Flag to build the filelists or not (with many files this may take some time)
        """

        self.connectors={}

        logger.info("campaign list " + ' '.join(self.camp.get_campaign_list()))
        logger.info("camp_name set {}".format(camp_name))

        description_dir = ROOT_DIR.parents[1] / "larda-description"

        self.camp.assign_campaign(camp_name)
        config_file=self.camp.CONFIGURATION_FILE
        cinfo_hand_down = {'coordinates': self.camp.COORDINATES,
                           'altitude': self.camp.ALTITUDE,
                           'location': self.camp.LOCATION,
                           'mira_azi_zero': self.camp.info_dict['mira_azi_zero']}
        logger.debug("config file {}".format(config_file))
        
        paraminformation = ParameterInfo.ParameterInfo(
            self.camp.config_dir, config_file, 
            cinfo_hand_down=cinfo_hand_down)
        starttime = time.time()
        logger.info("camp.VALID_SYSTEMS {}".format(self.camp.VALID_SYSTEMS))
        
        #if camp_name == 'LACROS_at_Leipzig':
        #    build_lists = False
        # build the filelists or load them from json
        for system, systeminfo in paraminformation.iterate_systems(keys=self.camp.VALID_SYSTEMS):
            logger.debug('current parameter {} {}'.format(system, systeminfo))

            if system in self.camp.system_only:
                valid_dates = self.camp.system_only[system]
            else:
                valid_dates = self.camp.VALID_DATES
            conn = Connector.Connector(system, systeminfo,
                                       valid_dates,
                                       description_dir=description_dir)
            
            if build_lists:
                conn.build_filehandler()
                conn.save_filehandler(self.camp.info_dict['connectordump'], camp_name)

            else:
                #load lists
                conn.load_filehandler(self.camp.info_dict['connectordump'], camp_name)
            
            if system in self.camp.VALID_SYSTEMS:
                self.connectors[system] = conn
            else:
                logger.warning("{} not in valid systems".format(system))

        logger.debug('Time for generating connectors {} s'.format(time.time() - starttime))
        #print "Availability array: ", self.array_avail(2014, 2)
        logger.warning(self.camp.INFO_TEXT)
        logger.info("systems {}".format(self.connectors.keys()))
        logger.info("Parameters in stock: {}".format([(k, self.connectors[k].params_list) for k in self.connectors.keys()]))
        return self
        

    def connect_remote(self, camp_name, **kwargs):
        logger.info("connect_remote {}".format(camp_name))
        resp = requests.get(self.uri + '/api/{}/'.format(camp_name))
        #print(resp.json())
        self.camp = LARDA_campaign_remote(resp.json()['config_file'])

        self.camp.INFO_TEXT = resp.json()['info_text']

        self.connectors = {}
        for k, c in resp.json()['connectors'].items():
            self.connectors[k] = Connector.Connector_remote(camp_name, k, c, self.uri)

        logger.warning(self.camp.INFO_TEXT)
        return self


    def read(self, system, parameter, time_interval, *further_slices, **kwargs):
        """
        Args:
            system (str): identifier for the system
            parameter (str): choosen param
            time_interval: ``[dt, dt]`` time interval, or [dt] one time
            *further_slices: range, vel,.. ``[0, max]`` or [3000]

        Returns:
            the dictionary with data
        """
        data = self.connectors[system].collect(parameter, time_interval, *further_slices, **kwargs) 

        return data

    def description(self, system, parameter):
        """
        Args:
            system (str): identifier for the system
            parameter (str): choosen param

        Returns:
            the description tag as string
        """
        descr = self.connectors[system].description(parameter)

        return descr


    def print_params(self):
        print("System, Param")
        [print(k, self.connectors[k].params_list) for k in self.connectors.keys()]

    def get_avail_dict(self, *args):
        """get the no files for each date
        Args:
            *args: either empty or `system`, `parameter`
        Returns:
            the number of files per day for each system
        """
        if len(args) == 0:
            return {system:conn.get_as_plain_dict() for system, conn in self.connectors.items()}
        elif len(args) == 2:
            system, param = args
            d = self.connectors[system].get_as_plain_dict()
            return d['avail'][d['params'][param]]


def resolve_today(lst):
    """ resolve 'today' in [['2012101', 'today']]"""
    return [[e[0], e[1]] if e[1] != 'today' 
            else [e[0], datetime.datetime.utcnow().strftime("%Y%m%d")]
            for e in lst
           ]


class LARDA_campaign:
    """ provides information about campaigns collected in LARDA"""
    def __init__(self, config_dir, campaign_file):

        logger.info('campaign file at LARDA_campaign ' + campaign_file)
        self.campaigns = toml.load(config_dir / campaign_file)
        self.campaigns = toml.load(Path(config_dir) / Path(campaign_file))
        self.campaign_list = list(self.campaigns.keys())
        self.config_dir = config_dir

    def get_campaign_list(self):
        """ list of all campaign names stored in csv-file """
        return self.campaign_list
        

    def assign_campaign(self, name):
        """dedicate object to a specific campaign"""
        
        self.info_dict = self.campaigns[name]
        logger.debug("info dict@assing campaign {}".format(self.info_dict))

        self.ALTITUDE=float(self.info_dict['altitude'])
        self.VALID_SYSTEMS = self.info_dict['systems']
        self.VALID_DATES = resolve_today(self.info_dict["duration"])
        self.COORDINATES = self.info_dict["coordinates"]
        self.CLOUDNET_STATIONNAME = self.info_dict["cloudnet_stationname"]
        self.CONFIGURATION_FILE = self.info_dict["param_config_file"]
        self.LOCATION = self.info_dict["location"]

        if not 'info_text_loc' in self.info_dict \
                or self.info_dict['info_text_loc'] == 'default':
            self.INFO_TEXT = __default_info__
        else:
            self.INFO_TEXT = toml.load(
                self.config_dir / self.info_dict['info_text_loc'])['info_text']

        if 'system_only' in self.info_dict:
            self.system_only = {
                k: resolve_today(v) for k, v in self.info_dict['system_only'].items()}
        else:
            self.system_only = {}


class LARDA_campaign_remote:
    """store the campaign information in a similar structure as for local"""
    def __init__(self, info_dict):

        self.info_dict = info_dict
        self.ALTITUDE=float(self.info_dict['altitude'])
        self.VALID_SYSTEMS = self.info_dict['systems']
        self.VALID_DATES = resolve_today(self.info_dict["duration"])
        self.COORDINATES = self.info_dict["coordinates"]
        self.CLOUDNET_STATIONNAME = self.info_dict["cloudnet_stationname"]
        self.CONFIGURATION_FILE = self.info_dict["param_config_file"]
        self.LOCATION = self.info_dict["location"]

        self.INFO_TEXT = ''




#!/usr/bin/python3

import datetime
import numpy as np
import netCDF4
import ast
import sys
sys.path.append('../')
sys.path.append('.')
import pyLARDA.helpers as h
import pyLARDA.Transformations as Transf
import pyLARDA.NcReader as NcReader

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

import logging
logger = logging.getLogger(__name__)


def trace_reader(paraminfo):
    """build a function for reading the trace_airmass_source data (setup by connector)"""
    def t_r(f, time_interval, *further_intervals):
        """function that converts the trace_airmass_source netCDF to the data container
        """
        logger.debug("filename at reader {}".format(f))
        with netCDF4.Dataset(f, 'r') as ncD:

            times = ncD.variables[paraminfo['time_variable']][:].astype(np.float64)

            timeconverter, _ = h.get_converter_array(
                paraminfo['time_conversion'], ncD=ncD)
            ts = timeconverter(times)

            #print('timestamps ', ts[:5])
            # setup slice to load base on time_interval
            it_b = h.argnearest(ts, h.dt_to_ts(time_interval[0]))
            if len(time_interval) == 2:
                it_e = h.argnearest(ts, h.dt_to_ts(time_interval[1]))
                if ts[it_e] < h.dt_to_ts(time_interval[0])-3*np.median(np.diff(ts)):
                    logger.warning(
                            'last profile of file {}\n at {} too far from {}'.format(
                                f, h.ts_to_dt(ts[it_e]), time_interval[0]))
                    return None

                it_e = it_e+1 if not it_e == ts.shape[0]-1 else None
                slicer = [slice(it_b, it_e)]
            else:
                slicer = [slice(it_b, it_b+1)]
            print(slicer)

            range_interval = further_intervals[0]
            ranges = ncD.variables[paraminfo['range_variable']]
            logger.debug('loader range conversion {}'.format(paraminfo['range_conversion']))
            rangeconverter, _ = h.get_converter_array(
                paraminfo['range_conversion'],
                altitude=paraminfo['altitude'])
            ir_b = h.argnearest(rangeconverter(ranges[:]), range_interval[0])
            if len(range_interval) == 2:
                if not range_interval[1] == 'max':
                    ir_e = h.argnearest(rangeconverter(ranges[:]), range_interval[1])
                    ir_e = ir_e+1 if not ir_e == ranges.shape[0]-1 else None
                else:
                    ir_e = None
                slicer.append(slice(ir_b, ir_e))
            else:
                slicer.append(slice(ir_b, ir_b+1))

            varconverter, maskconverter = h.get_converter_array(
                paraminfo['var_conversion'])

            its = np.arange(ts.shape[0])[tuple(slicer)[0]]
            irs = np.arange(ranges.shape[0])[tuple(slicer)[1]]
            var = np.empty((its.shape[0], irs.shape[0]))
            mask = np.empty((its.shape[0], irs.shape[0]))
            mask[:] = False

            no_occ = ncD.variables[paraminfo['variable_name'] + "_no_below"][tuple(slicer)[0],tuple(slicer)[1]]
            occ_height = ncD.variables[paraminfo['variable_name']][tuple(slicer)[0],tuple(slicer)[1],:]
            var = occ_height*no_occ[:, :, np.newaxis]

            data = {}
            data['dimlabel'] = ['time', 'range', 'cat']

            data["filename"] = f
            data["paraminfo"] = paraminfo
            data['ts'] = ts[tuple(slicer)[0]]

            data['system'] = paraminfo['system']
            data['name'] = paraminfo['paramkey']
            data['colormap'] = paraminfo['colormap']

            if 'meta' in paraminfo:
                data['meta'] = NcReader.get_meta_from_nc(ncD, paraminfo['meta'], paraminfo['variable_name'])

            variable = ncD.variables[paraminfo['variable_name']]
            var_definition = ast.literal_eval(
                variable.getncattr(paraminfo['identifier_var_def']))
            if var_definition[1] == "forrest":
                var_definition[1] = "forest"

            data['var_definition'] = var_definition

            data['rg'] = rangeconverter(ranges[tuple(slicer)[1]])
            data['rg_unit'] = NcReader.get_var_attr_from_nc("identifier_rg_unit", 
                                                paraminfo, ranges)
            logger.debug('shapes {} {} {}'.format(ts.shape, ranges.shape, var.shape))

            data['var_unit'] = NcReader.get_var_attr_from_nc("identifier_var_unit", 
                                                    paraminfo, var)
            data['var_lims'] = [float(e) for e in \
                                NcReader.get_var_attr_from_nc("identifier_var_lims", 
                                                    paraminfo, var)]

            data['var'] = varconverter(var)
            data['mask'] = maskconverter(mask)

            return data

    return t_r


def plot_ls_2d(airmass_source, plottop=12000, xright=7000, xlbl_time='%H:%M', norm=None):
    #f, parameter, dt_list, dsp, config, savepath, config_dict, model):
    time_list = airmass_source['ts']
    dt_list = [h.ts_to_dt(time) for time in time_list]
    height_list = airmass_source['rg']/1000.
    no_plots = len(dt_list)

    if no_plots > 9:
        xsize = 15
    elif no_plots < 8:
        xsize = no_plots*1.8
    else:
        xsize = 12
    
    if norm:
        airmass_source['var'][:,:,:] = airmass_source['var'][:,:,:].copy()/norm
        dsp_text = 'Norm. residence time'
    else:
        dsp_text = 'acc. residence time'

    axes = []
    fig = plt.figure(constrained_layout=False,
                     figsize=(xsize, 6))
    gs1 = fig.add_gridspec(nrows=9, ncols=no_plots, wspace=0.0)
    for i in range(no_plots):
        axes.append(fig.add_subplot(gs1[1:, i]))
    #fig, axes = plt.subplots(1, no_plots, sharex=True, sharey=True, figsize=(9, 6))
    ls_colors = ['lightskyblue', 'darkgreen', 'khaki', 'palegreen', 'red', 'gray', 'tan']
    ls_colors = ['lightskyblue', 'seagreen', 'khaki', '#6edd6e', 'darkmagenta', 'gray', 'tan']

    for it, dt in enumerate(dt_list):

        occ_height = airmass_source['var'][it,:,:]
        occ_left = np.cumsum(occ_height, axis=1)

        categories = airmass_source['var_definition']

        l1 = axes[it].barh(height_list, occ_height[:, 0].T, 
                           align='center', height=0.35, color=ls_colors[0], edgecolor='none')
        l2 = axes[it].barh(height_list, occ_height[:, 1].T, left=occ_left[:, 0].T,
                           align='center', height=0.35, color=ls_colors[1], edgecolor='none')
        l3 = axes[it].barh(height_list, occ_height[:, 2].T, left=occ_left[:, 1].T,
                           align='center', height=0.35, color=ls_colors[2], edgecolor='none')
        l4 = axes[it].barh(height_list, occ_height[:, 3].T, left=occ_left[:, 2].T,
                           align='center', height=0.35, color=ls_colors[3], edgecolor='none')
        l5 = axes[it].barh(height_list, occ_height[:, 4].T, left=occ_left[:, 3].T,
                           align='center', height=0.35, color=ls_colors[4], edgecolor='none')
        l6 = axes[it].barh(height_list, occ_height[:, 5].T, left=occ_left[:, 4].T,
                           align='center', height=0.35, color=ls_colors[5], edgecolor='none')

        l7 = axes[it].barh(height_list, occ_height[:, 6].T, left=occ_left[:, 5].T,
                          align='center', height=0.35, color=ls_colors[6], edgecolor='none')

        #axes[it].set_ylim([0, 12])
        axes[it].set_ylim([0, plottop/1000.])
        axes[it].tick_params(axis='y', which='major', labelsize=14, 
                             width=1.5, length=3)
        axes[it].tick_params(axis='both', which='minor', width=1, length=2)
        axes[it].tick_params(axis='both', which='both', right=True, top=True,
                             direction='in')
        axes[it].yaxis.set_minor_locator(matplotlib.ticker.MultipleLocator(1.0))
        axes[it].set_xlabel(dt.strftime(xlbl_time), fontsize=14)
        
        axes[it].set_xlim(right=xright)
        if xright < 20000:
            axes[it].xaxis.set_major_locator(matplotlib.ticker.MultipleLocator(3000))
        else:
            axes[it].xaxis.set_major_locator(matplotlib.ticker.MaxNLocator(nbins=1))
        if norm:
            axes[it].xaxis.set_major_locator(matplotlib.ticker.AutoLocator())
        axes[it].xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
        axes[it].tick_params(axis='x', labeltop=False, labelbottom=False)
        axes[it].tick_params(axis='y', labelleft=False)

    axes[0].set_ylabel("Height [km]", fontweight='semibold', fontsize=15)
    axes[0].tick_params(axis='y', labelleft=True)
    axes[-1].tick_params(axis='x', labeltop=True, labelbottom=False, labelsize=13)
    
    plt.suptitle("{}   {}   {}".format(dt.strftime("%Y%m%d"), airmass_source['paraminfo']['location'], airmass_source['name']), 
                 fontweight='semibold', fontsize=15)


    axes[-1].annotate("Endpoint: {:.1f} {:.1f} ".format(airmass_source['paraminfo']['coordinates'][1], 
                                                        airmass_source['paraminfo']['coordinates'][0]), 
                        xy=(.88, 0.92), xycoords='figure fraction',
                        horizontalalignment='center', verticalalignment='bottom',
                        fontsize=12)
    axes[0].annotate('Time UTC', xy=(.5, .02),
                     xycoords='figure fraction',
                     horizontalalignment='center', verticalalignment='bottom',
                     fontsize=15, fontweight='semibold')

    axes[-1].annotate(dsp_text, xy=(.87, 0.845),
                      xycoords='figure fraction',
                      horizontalalignment='center', verticalalignment='bottom',
                      fontsize=13, fontweight='semibold')
    fig.legend((l1, l2, l3, l4, l5, l6, l7), list(categories.values()),
               #bbox_to_anchor=(0.85, 0.952),
               bbox_to_anchor=(0.07, 0.94), loc='upper left',
               columnspacing=1,
               ncol=4, fontsize=14, framealpha=0.8)

    #fig.set_tight_layout({'rect': [0, 0, 1, 1], 'pad': 0.1, 'h_pad': 1.5})
    #plt.tight_layout(w_pad=0.0002)

    
    return fig, axes


def plot_gn_2d(airmass_source, plottop=12000, xright=7000, xlbl_time='%H:%M', norm=None):
    #f, parameter, dt_list, dsp, config, savepath, config_dict, model):
    time_list = airmass_source['ts']
    dt_list = [h.ts_to_dt(time) for time in time_list]
    height_list = airmass_source['rg']/1000.
    no_plots = len(dt_list)

    if no_plots > 9:
        xsize = 15
    elif no_plots < 8:
        xsize = no_plots*1.8
    else:
        xsize = 12


    if norm:
        airmass_source['var'][:,:,:] = airmass_source['var'][:,:,:].copy()/norm
        dsp_text = 'Norm. residence time'
    else:
        dsp_text = 'acc. residence time'
        
#     fig, axes = plt.subplots(1, no_plots, sharex=True, sharey=True, 
#                              figsize=(xsize, 6))
    axes = []
    fig = plt.figure(constrained_layout=False,
                     figsize=(xsize, 6))
    gs1 = fig.add_gridspec(nrows=9, ncols=no_plots, wspace=0.0)
    for i in range(no_plots):
        axes.append(fig.add_subplot(gs1[1:, i]))
    

    colors = [(0.65098039215686276, 0.84705882352941175, 0.32941176470588235, 1.0), 
              (1.0, 0.85098039215686272, 0.18431372549019609, 1.0), 
              (0.89803921568627454, 0.7686274509803922, 0.58039215686274515, 1.0),
              (0.40000000000000002, 0.76078431372549016, 0.6470588235294118, 1.0), 
              (0.9882352941176471, 0.55294117647058827, 0.3843137254901961, 1.0), 
              (0.55294117647058827, 0.62745098039215685, 0.79607843137254897, 1.0),  
              (0.70196078431372544, 0.70196078431372544, 0.70196078431372544, 1.0)]

    for it, dt in enumerate(dt_list):

        occ_height = airmass_source['var'][it,:,:]
        occ_left = np.cumsum(occ_height, axis=1)

        geo_names = airmass_source['var_definition']

        l = []
        for i in range(len(geo_names)):
            if i == 0:
                l.append(axes[it].barh(height_list, occ_height[:, 0].T, left=0, 
                    align='center', height=0.3, color=colors[0], edgecolor='none'))
            else:
                l.append(axes[it].barh(height_list, occ_height[:, i].T, left=occ_left[:, i-1].T,
                    align='center', height=0.3, color=colors[i], edgecolor='none'))

        #axes[it].set_ylim([0, 12])
        axes[it].set_ylim([0, plottop/1000.])
        axes[it].tick_params(axis='y', which='major', labelsize=14, 
                             width=1.5, length=3)
        axes[it].tick_params(axis='both', which='minor', width=1, length=2)
        axes[it].tick_params(axis='both', which='both', right=True, top=True,
                             direction='in')
        axes[it].yaxis.set_minor_locator(matplotlib.ticker.MultipleLocator(1.0))
        axes[it].set_xlabel(dt.strftime(xlbl_time), fontsize=14)
        
        axes[it].set_xlim([0, xright])        
        if xright < 20000:
            axes[it].xaxis.set_major_locator(matplotlib.ticker.MultipleLocator(3000))
        else:
            # axes[it].xaxis.set_major_locator(matplotlib.ticker.FixedLocator([0, xright/2.]))
            axes[it].xaxis.set_major_locator(matplotlib.ticker.MaxNLocator(nbins=1))
        if norm:
            axes[it].xaxis.set_major_locator(matplotlib.ticker.AutoLocator())
        axes[it].xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
        axes[it].tick_params(axis='x', labeltop=False, labelbottom=False)
        axes[it].tick_params(axis='y', labelleft=False)

    axes[0].set_ylabel("Height [km]", fontweight='semibold', fontsize=15)
    axes[0].tick_params(axis='y', labelleft=True)
    axes[-1].tick_params(axis='x', labeltop=True, labelbottom=False, labelsize=13)
    
    plt.suptitle("{}   {}   {}".format(dt.strftime("%Y%m%d"), airmass_source['paraminfo']['location'], airmass_source['name']), 
                 fontweight='semibold', fontsize=15)


    axes[-1].annotate("Endpoint: {:.1f} {:.1f} ".format(airmass_source['paraminfo']['coordinates'][1], 
                                                        airmass_source['paraminfo']['coordinates'][0]), 
                        xy=(.88, 0.92), xycoords='figure fraction',
                        horizontalalignment='center', verticalalignment='bottom',
                        fontsize=12)
    axes[0].annotate('Time UTC', xy=(.5, .02),
                     xycoords='figure fraction',
                     horizontalalignment='center', verticalalignment='bottom',
                     fontsize=15, fontweight='semibold')

    axes[-1].annotate(dsp_text, xy=(.87, 0.845),
                      xycoords='figure fraction',
                      horizontalalignment='center', verticalalignment='bottom',
                      fontsize=13, fontweight='semibold')
    fig.legend(l, list(geo_names.values()),
               loc='upper left',
               #bbox_to_anchor=(0.01, 0.952),
               bbox_to_anchor=(0.07, 0.94),
               columnspacing=1,
               ncol=4, fontsize=14, framealpha=0.8)

    return fig, axes



def trace_reader2(paraminfo):
    """build a function for reading the new trace format (since Dec 2020) data (setup by connector)"""
    def t_r(f, time_interval, *further_intervals):
        """function that converts the trace netCDF to the data container
        """
        logger.debug("filename at reader {}".format(f))
        with netCDF4.Dataset(f, 'r') as ncD:

            times = ncD.variables[paraminfo['time_variable']][:].astype(np.float64)

            timeconverter, _ = h.get_converter_array(
                paraminfo['time_conversion'], ncD=ncD)
            ts = timeconverter(times)

            #print('timestamps ', ts[:5])
            # setup slice to load base on time_interval
            it_b = h.argnearest(ts, h.dt_to_ts(time_interval[0]))
            if len(time_interval) == 2:
                it_e = h.argnearest(ts, h.dt_to_ts(time_interval[1]))
                if ts[it_e] < h.dt_to_ts(time_interval[0])-3*np.median(np.diff(ts)):
                    logger.warning(
                            'last profile of file {}\n at {} too far from {}'.format(
                                f, h.ts_to_dt(ts[it_e]), time_interval[0]))
                    return None

                it_e = it_e+1 if not it_e == ts.shape[0]-1 else None
                slicer = [slice(it_b, it_e)]
            else:
                slicer = [slice(it_b, it_b+1)]
            print(slicer)

            range_interval = further_intervals[0]
            ranges = ncD.variables[paraminfo['range_variable']]
            logger.debug('loader range conversion {}'.format(paraminfo['range_conversion']))
            rangeconverter, _ = h.get_converter_array(
                paraminfo['range_conversion'],
                altitude=paraminfo['altitude'])
            ir_b = h.argnearest(rangeconverter(ranges[:]), range_interval[0])
            if len(range_interval) == 2:
                if not range_interval[1] == 'max':
                    ir_e = h.argnearest(rangeconverter(ranges[:]), range_interval[1])
                    ir_e = ir_e+1 if not ir_e == ranges.shape[0]-1 else None
                else:
                    ir_e = None
                slicer.append(slice(ir_b, ir_e))
            else:
                slicer.append(slice(ir_b, ir_b+1))

            varconverter, maskconverter = h.get_converter_array(
                paraminfo['var_conversion'])

            its = np.arange(ts.shape[0])[tuple(slicer)[0]]
            irs = np.arange(ranges.shape[0])[tuple(slicer)[1]]
            var = np.empty((its.shape[0], irs.shape[0]))
            mask = np.empty((its.shape[0], irs.shape[0]))
            mask[:] = False

            var = ncD.variables[paraminfo['variable_name']][tuple(slicer)[0],tuple(slicer)[1],:]

            data = {}
            data['dimlabel'] = ['time', 'range', 'cat']

            data["filename"] = f
            data["paraminfo"] = paraminfo
            data['ts'] = ts[tuple(slicer)[0]]

            data['system'] = paraminfo['system']
            data['name'] = paraminfo['paramkey']
            data['colormap'] = paraminfo['colormap']

            if 'meta' in paraminfo:
                data['meta'] = NcReader.get_meta_from_nc(ncD, paraminfo['meta'], paraminfo['variable_name'])

            variable = ncD.variables[paraminfo['variable_name']]
            var_definition = ast.literal_eval(
                variable.getncattr(paraminfo['identifier_var_def']))
            if var_definition[1] == "forrest":
                var_definition[1] = "forest"

            data['var_definition'] = var_definition

            data['rg'] = rangeconverter(ranges[tuple(slicer)[1]])
            data['rg_unit'] = NcReader.get_var_attr_from_nc("identifier_rg_unit", 
                                                paraminfo, ranges)
            logger.debug('shapes {} {} {}'.format(ts.shape, ranges.shape, var.shape))

            data['var_unit'] = NcReader.get_var_attr_from_nc("identifier_var_unit", 
                                                    paraminfo, var)
            data['var_lims'] = [float(e) for e in \
                                NcReader.get_var_attr_from_nc("identifier_var_lims", 
                                                    paraminfo, var)]

            data['var'] = varconverter(var)
            data['mask'] = maskconverter(mask)

            return data

    return t_r



#!/usr/bin/python3

import sys
# just needed to find pyLARDA from this location
sys.path.append('../')
sys.path.append('.')


import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

import pyLARDA
import pyLARDA.helpers as h
import datetime
import numpy as np
import scipy.ndimage as spn
from scipy import stats

#Load LARDA
#larda=pyLARDA.LARDA('lacros_dacapo')

larda=pyLARDA.LARDA().connect_local('lacros_dacapo')
#c_info = [larda.camp.LOCATION, larda.camp.VALID_DATES]

print(larda.days_with_data())
#print("array_avail()", larda.array_avail(2015, 6))
#print("single month with new interface ", larda.instr_status(2015, 6)) 


#begin_dt=datetime.datetime(2018,12,6,0,1)
begin_dt=datetime.datetime(2018,12,6,1,40)
end_dt=datetime.datetime(2018,12,6,4,0,0)

plot_range = [300, 10000]
case_prefix = 'plots/scatter_case_studies/comparison_cloud_'

# load the reflectivity data
MIRA_Zg=larda.read("MIRA","Zg",[begin_dt,end_dt],[0,'max'])
MIRA_Zg['var_lims'] = [-40,20]
fig, ax = pyLARDA.Transformations.plot2d(MIRA_Zg, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'mira_z.png', dpi=250)


LIMRAD94_Z=larda.read("LIMRAD94","Ze",[begin_dt,end_dt],[0,'max'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_Z, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'limrad_Z.png', dpi=250)

LIMRAD94_Z_interp = pyLARDA.Transformations.interpolate2d(LIMRAD94_Z, new_time=MIRA_Zg['ts'], new_range=MIRA_Zg['rg'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_Z_interp, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'limrad_Z_interp.png', dpi=250)

# Z scatterplot

combined_mask = np.logical_or(MIRA_Zg['mask'], LIMRAD94_Z_interp['mask'])

Mira_Z_scatter = h.lin2z(MIRA_Zg['var'][~combined_mask].ravel())#+4.5
Limrad_Z_scatter = h.lin2z(LIMRAD94_Z_interp['var'][~combined_mask].ravel())

s, i, r, p, std_err = stats.linregress(Mira_Z_scatter, Limrad_Z_scatter)
H, xedges, yedges = np.histogram2d(Mira_Z_scatter, Limrad_Z_scatter, 
                                   bins=120, range=[[-75, 30], [-75, 30]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)

# helper lines (1:1), ...
ax.plot([-80,30],[-80,30], color='salmon')
ax.plot([-80,30],[-90,20], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-85,25], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-75,35], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-70,40], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-75,20])
ax.set_ylim([-75,20])
ax.set_xlabel("MIRA Z")
ax.set_ylabel("LIMRAD94 Z")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_mira_limrad.png', dpi=250)

# correct the found bias
def bias(datalist):
    var = datalist[0]['var'].copy() * h.z2lin(4.7)
    mask = datalist[0]['mask']
    return var, mask
MIRA_Zg = pyLARDA.Transformations.combine(bias, [MIRA_Zg], MIRA_Zg['paraminfo'])
fig, ax = pyLARDA.Transformations.plot2d(MIRA_Zg, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'mira_z_bias_4-7.png', dpi=250)

# calc and plot the DWR
def calc_DWR(datalist):
    var = h.lin2z(datalist[0]['var']) - h.lin2z(datalist[1]['var'])
    mask = np.logical_or(datalist[0]['mask'], datalist[1]['mask'])
    return var, mask
pinfo = {'system': '', 'name': 'DWR', 'colormap': 'gist_rainbow', 'rg_unit': 'm', 'var_lims': [-3, 7]}
DWR = pyLARDA.Transformations.combine(calc_DWR, [MIRA_Zg, LIMRAD94_Z_interp], pinfo)
fig, ax = pyLARDA.Transformations.plot2d(DWR, range_interval=plot_range)
fig.savefig(case_prefix+'DWR_MIRA_LIMRAD.png', dpi=250)

# other day for the mrr

begin_dt=datetime.datetime(2018,12,9,2,0)
end_dt=datetime.datetime(2018,12,9,5,0,0)
case_prefix = 'plots/scatter_case_studies/comparison_rain_'

MIRA_Zg=larda.read("MIRA","Zg",[begin_dt,end_dt],[0,4000])
MIRA_Zg['var_lims'] = [-40,20]
fig, ax = pyLARDA.Transformations.plot2d(MIRA_Zg, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'mira_z.png', dpi=250)


LIMRAD94_Z=larda.read("LIMRAD94","Ze",[begin_dt,end_dt],[0,4000])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_Z, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'limrad_Z.png', dpi=250)
LIMRAD94_Z_interp = pyLARDA.Transformations.interpolate2d(LIMRAD94_Z, new_time=MIRA_Zg['ts'], new_range=MIRA_Zg['rg'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_Z_interp, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'limrad_Z_interp.png', dpi=250)

combined_mask = np.logical_or(MIRA_Zg['mask'], LIMRAD94_Z_interp['mask'])

Mira_Z_scatter = h.lin2z(MIRA_Zg['var'][~combined_mask].ravel())#+4.5
Limrad_Z_scatter = h.lin2z(LIMRAD94_Z_interp['var'][~combined_mask].ravel())

s, i, r, p, std_err = stats.linregress(Mira_Z_scatter, Limrad_Z_scatter)
H, xedges, yedges = np.histogram2d(Mira_Z_scatter, Limrad_Z_scatter, 
                                   bins=120, range=[[-75, 30], [-75, 30]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)
ax.plot([-80,30],[-80,30], color='salmon')
ax.plot([-80,30],[-90,20], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-85,25], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-75,35], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-70,40], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-75,20])
ax.set_ylim([-75,20])
ax.set_xlabel("MIRA Z")
ax.set_ylabel("LIMRAD94 Z")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_mira_limrad.png', dpi=250)


MRR_Z=larda.read("MRRPRO","Ze",[begin_dt,end_dt],[0,4000])
fig, ax = pyLARDA.Transformations.plot2d(MRR_Z, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'mrr_Z.png', dpi=250)
MRR_Z_interp = pyLARDA.Transformations.interpolate2d(MRR_Z, new_time=MIRA_Zg['ts'], new_range=MIRA_Zg['rg'])
fig, ax = pyLARDA.Transformations.plot2d(MRR_Z_interp, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'mrr_Z_interp.png', dpi=250)

# scatterplot
combined_mask = np.logical_or(MIRA_Zg['mask'], MRR_Z_interp['mask'])

Mira_Z_scatter = h.lin2z(MIRA_Zg['var'][~combined_mask].ravel())#+4.5
Mrr_Z_scatter = h.lin2z(MRR_Z_interp['var'][~combined_mask].ravel())

s, i, r, p, std_err = stats.linregress(Mira_Z_scatter, Mrr_Z_scatter)
H, xedges, yedges = np.histogram2d(Mira_Z_scatter, Mrr_Z_scatter, 
                                   bins=120, range=[[-75, 30], [-75, 30]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)
ax.plot([-80,30],[-80,30], color='salmon')
ax.plot([-80,30],[-90,20], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-85,25], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-75,35], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-70,40], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-75,20])
ax.set_ylim([-75,20])
ax.set_xlabel("MIRA Z")
ax.set_ylabel("MRR Z")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_mira_MRR.png', dpi=250)


MRR_Z_interp = pyLARDA.Transformations.interpolate2d(MRR_Z, new_time=LIMRAD94_Z['ts'], new_range=LIMRAD94_Z['rg'])
fig, ax = pyLARDA.Transformations.plot2d(MRR_Z_interp, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'mrr_Z_interp_to_limrad.png', dpi=250)

combined_mask = np.logical_or(LIMRAD94_Z['mask'], MRR_Z_interp['mask'])

Limrad_Z_scatter = h.lin2z(LIMRAD94_Z['var'][~combined_mask].ravel())#+4.5
Mrr_Z_scatter = h.lin2z(MRR_Z_interp['var'][~combined_mask].ravel())

s, i, r, p, std_err = stats.linregress(Limrad_Z_scatter, Mrr_Z_scatter)
H, xedges, yedges = np.histogram2d(Limrad_Z_scatter, Mrr_Z_scatter, 
                                   bins=120, range=[[-75, 30], [-75, 30]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)
ax.plot([-80,30],[-80,30], color='salmon')
ax.plot([-80,30],[-90,20], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-85,25], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-75,35], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-70,40], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-75,20])
ax.set_ylim([-75,20])
ax.set_xlabel("LIMRAD Z")
ax.set_ylabel("MRR Z")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_limrad_MRR.png', dpi=250)



#begin_dt=datetime.datetime(2018,12,6,0,1)
begin_dt=datetime.datetime(2018,12,14,6,0)
end_dt=datetime.datetime(2018,12,14,9,0)

plot_range = [300, 10000]
case_prefix = 'plots/scatter_case_studies/comparison_c14_'

# load the reflectivity data
MIRA_Zg=larda.read("MIRA","Zg",[begin_dt,end_dt],[0,'max'])
MIRA_Zg['var_lims'] = [-40,20]
fig, ax = pyLARDA.Transformations.plot2d(MIRA_Zg, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'mira_z.png', dpi=250)


LIMRAD94_Z=larda.read("LIMRAD94","Ze",[begin_dt,end_dt],[0,'max'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_Z, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'limrad_Z.png', dpi=250)

LIMRAD94_Z_interp = pyLARDA.Transformations.interpolate2d(LIMRAD94_Z, new_time=MIRA_Zg['ts'], new_range=MIRA_Zg['rg'])
fig, ax = pyLARDA.Transformations.plot2d(LIMRAD94_Z_interp, range_interval=plot_range, z_converter='lin2z')
fig.savefig(case_prefix+'limrad_Z_interp.png', dpi=250)

# Z scatterplot

combined_mask = np.logical_or(MIRA_Zg['mask'], LIMRAD94_Z_interp['mask'])

Mira_Z_scatter = h.lin2z(MIRA_Zg['var'][~combined_mask].ravel())#+4.5
Limrad_Z_scatter = h.lin2z(LIMRAD94_Z_interp['var'][~combined_mask].ravel())

s, i, r, p, std_err = stats.linregress(Mira_Z_scatter, Limrad_Z_scatter)
H, xedges, yedges = np.histogram2d(Mira_Z_scatter, Limrad_Z_scatter, 
                                   bins=120, range=[[-75, 30], [-75, 30]])
X, Y = np.meshgrid(xedges,yedges)
fig, ax = plt.subplots(1, figsize=(5.7, 5.7))
ax.pcolormesh(X, Y, np.transpose(H),
              norm=matplotlib.colors.LogNorm(),
)
ax.text(0.01, 0.93, 'slope {:5.3f}\nintercept {:5.3f}\nR^2 {:5.3f}'.format(s,i,r**2), 
        horizontalalignment='left',
        verticalalignment='center', transform=ax.transAxes)
#ax.scatter(Mira_Z_scatter, Limrad_Z_scatter, s=1)
ax.plot([-80,30],[-80,30], color='salmon')
ax.plot([-80,30],[-90,20], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-85,25], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-75,35], color='salmon', linewidth=0.7, linestyle='--')
ax.plot([-80,30],[-70,40], color='salmon', linewidth=0.7, linestyle='--')
ax.set_xlim([-75,20])
ax.set_ylim([-75,20])
ax.set_xlabel("MIRA Z")
ax.set_ylabel("LIMRAD94 Z")
ax.xaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.yaxis.set_minor_locator(matplotlib.ticker.AutoMinorLocator())
ax.tick_params(axis='both', which='both', right=True, top=True)
fig.savefig(case_prefix+'scatter_mira_limrad.png', dpi=250)



#!/usr/bin/python3

import sys
# just needed to find pyLARDA from this location
sys.path.append('../')
sys.path.append('.')


import matplotlib
matplotlib.use('Agg')
import pyLARDA
import pyLARDA.helpers as h
import datetime
import numpy as np
import scipy.ndimage as spn

import logging
log = logging.getLogger('pyLARDA')
log.setLevel(logging.INFO)
log.addHandler(logging.StreamHandler())

#Load LARDA
larda=pyLARDA.LARDA().connect('lacros_dacapo')
c_info = [larda.camp.LOCATION, larda.camp.VALID_DATES]

print(larda.days_with_data())
#print("array_avail()", larda.array_avail(2015, 6))
#print("single month with new interface ", larda.instr_status(2015, 6)) 


begin_dt=datetime.datetime(2018,12,6,0,1)
end_dt=datetime.datetime(2018,12,6,4,0,0)

plot_range = [300, 10000]

MIRA_Zg=larda.read("MIRA","Zg",[begin_dt,end_dt],[0,'max'])
MIRA_Zg['var_lims'] = [-40,20]
#fig, ax = pyLARDA.Transformations.plot_timeheight(params["MIRA_Zg"],begin,end,plot_base,plot_top,savename="example/mira_zg.png")
fig, ax = pyLARDA.Transformations.plot_timeheight(MIRA_Zg, range_interval=plot_range, z_converter='lin2z')
fig.savefig('mira_z.png', dpi=250)


LIMRAD94_Z=larda.read("LIMRAD94","Ze",[begin_dt,end_dt],[0,'max'])
fig, ax = pyLARDA.Transformations.plot_timeheight(LIMRAD94_Z, range_interval=plot_range, z_converter='lin2z')
fig.savefig('limrad_Z.png', dpi=250)
#with np.printoptions(precision=12):
#    print("timestamps mira", MIRA_Zg['ts'].shape, MIRA_Zg['ts'].astype(float))
#    print("timesdiff mira", np.diff(MIRA_Zg['ts']))
#    print("timestamps limrad", MIRA_Zg['ts'].shape, LIMRAD94_Z['ts'].astype(float))
#    print("timesdiff limrad", np.diff(LIMRAD94_Z['ts']))
LIMRAD94_Z_interp = pyLARDA.Transformations.interpolate2d(LIMRAD94_Z, new_time=MIRA_Zg['ts'], new_range=MIRA_Zg['rg'])
fig, ax = pyLARDA.Transformations.plot_timeheight(LIMRAD94_Z_interp, range_interval=plot_range, z_converter='lin2z')
fig.savefig('limrad_Z_interp.png', dpi=250)

def calc_DWR(datalist):
    var = h.lin2z(datalist[0]['var']) - h.lin2z(datalist[1]['var'])
    mask = np.logical_or(datalist[0]['mask'], datalist[1]['mask'])
    return var, mask

new_keys = {'system': '', 'name': 'DWR', 'colormap': 'gist_rainbow', 'rg_unit': 'm', 'var_lims': [-7, 2]}
DWR = pyLARDA.Transformations.combine(calc_DWR, [MIRA_Zg, LIMRAD94_Z_interp], new_keys)
fig, ax = pyLARDA.Transformations.plot_timeheight(DWR, range_interval=plot_range)
fig.savefig('DWR_MIRA_LIMRAD.png', dpi=250)

MRR_Z=larda.read("MRRPRO","Ze",[begin_dt,end_dt],[0,'max'])
MRR_Z['var'] = h.lin2z(MRR_Z['var'])
fig, ax = pyLARDA.Transformations.plot_timeheight(MRR_Z, range_interval=plot_range)
fig.savefig('mrr_Z.png', dpi=250)

